declare module '@svelte-put/async-stack' {
	import type { Component, ComponentProps } from 'svelte';
	import type { ActionReturn } from 'svelte/action';
	export function stack(init?: StackItemCommonConfig<string, import("svelte").Component<any, any, string>> | undefined): StackBuilder;

	class StackBuilder<VariantMap extends Record<string, import("svelte").Component<any>> = {}> {
		
		constructor(init?: StackItemCommonConfig<string, import("svelte").Component<any, any, string>> | undefined);
		/**
		 * add config for a stack item variant
		 * */
		addVariant<Variant extends string, UserComponent extends import("svelte").Component<any>>(variant: Variant, config: UserComponent | Omit<StackItemVariantConfig<Variant, UserComponent>, "variant">): StackBuilder<VariantMap & Record<Variant, UserComponent>>;
		/**
		 * Build the actual stack
		 * */
		build(): Stack<VariantMap>;
		#private;
	}
	export class Stack<VariantMap extends Record<string, import("svelte").Component<any>> = {}> {
		
		constructor(variantConfigMap: Record<keyof VariantMap, StackItemVariantConfig<string, import("svelte").Component>>, init?: StackItemCommonConfig<string, import("svelte").Component<any, any, string>> | undefined);
		/**
		 * the stack items
		 * */
		items: StackItem<any>[];
		
		config: Required<StackItemCommonConfig<string, import("svelte").Component<any>>>;
		actions: {
			/**
			 * register the element to render a stack item into
			 * */
			render: (node: HTMLElement, item: StackItem<any>) => StackItemRenderActionReturn;
		};
		
		push<Variant extends Extract<keyof VariantMap, string>, UserComponent extends VariantMap[Variant] = VariantMap[Variant]>(variant: Variant, config?: StackItemByVariantPushConfig<Variant, UserComponent> | undefined): StackItem<UserComponent>;
		
		push<Variant extends Extract<keyof VariantMap, string>, UserComponent extends VariantMap[Variant] = VariantMap[Variant]>(variant: "custom", config: StackItemCustomPushConfig<import("svelte").Component<any>>): StackItem<import("svelte").Component<any>>;
		
		pop<UserComponent extends import("svelte").Component<any> = import("svelte").Component<any, any, string>>(id?: string | undefined, detail?: any): StackItem<UserComponent> | null;
		
		pop<UserComponent extends import("svelte").Component<any> = import("svelte").Component<any, any, string>>(config?: StackItemPopVerboseInput<UserComponent> | undefined): StackItem<UserComponent> | null;
		/**
		 * pause a stack item, if it has a timeout. If no `id` is provided,
		 * will pause all stack items with a timeout
		 * */
		pause(id?: string | undefined): void;
		/**
		 * resume a stack item, if it has been paused. If no `id` is provided,
		 * will resume all stack items with a timeout
		 * */
		resume(id?: string | undefined): void;
		#private;
	}
	export class StackItem<UserComponent extends import("svelte").Component<any> = import("svelte").Component<any, any, string>, Resolved = ComponentResolved<UserComponent>> {
		
		constructor(config: Required<StackItemInstanceConfig<string, UserComponent>>);
		
		state: StackItemState;
		
		config: Required<StackItemInstanceConfig<string, UserComponent>>;
		/**
		 * a promise that resolves when the item is popped or resolved via the .resolve method
		 * */
		resolution: Promise<Resolved | undefined>;
		resume: () => void;
		pause: () => void;
		
		resolve: (resolved?: Resolved | undefined) => Promise<Resolved | undefined>;
		#private;
	}
	export interface StackItemProps<Resolved = undefined> {
		/** the stack item instance injected by the stack */
		item: Omit<StackItem<Component<StackItemProps<Resolved>>, Resolved>, '#internals'>;
	}

	export type ComponentResolved<UserComponent extends Component<any>> =
		ComponentProps<UserComponent> extends { item: { resolution: Promise<infer Resolved> } }
			? Resolved
			: any;
	type StackItemCommonConfig<Variant extends string, UserComponent extends Component<any>> = {
		/**
		 * milliseconds to wait and automatically pop the stack item.
		 * Defaults to `0` (disabled)
		 */
		timeout?: number;
		/**
		 * id generator for stack items. Defaults to 'uuid'
		 *
		 *   - counter: use an auto-incremented counter that is scoped to the
		 *   - uuid: use `crypto.randomUUID()`, fallback to `counter` if not available
		 *   - callback: a custom function that accepts {@link StackItemInstanceConfig} and returns a string as the id
		 */
		id?:
			| 'counter'
			| 'uuid'
			| ((config: Required<Omit<StackItemInstanceConfig<Variant, UserComponent>, 'id'>>) => string);
	};

	/** predefined variant config provided while building a {@link Stack} instance */
	type StackItemVariantConfig<
		Variant extends string,
		UserComponent extends Component<any>,
	> = StackItemCommonConfig<Variant, UserComponent> & {
		/** string variant representing this config, must be unique within a {@link Stack} instance  */
		variant: Variant;
		/** any Svelte component used for rendering stack item UI */
		component: UserComponent;
		/** inferred props from `component` */
		props?: Omit<ComponentProps<UserComponent>, 'item'>;
	};

	/** a resolved config for a {@link StackItemInstance} */
	type StackItemInstanceConfig<
		Variant extends string,
		UserComponent extends Component<any>,
	> = Required<Omit<StackItemVariantConfig<Variant, UserComponent>, 'id'>> & {
		id: string;
		timeout: number;
	};

	type StackItemState = 'idle' | 'elapsing' | 'paused' | 'timeout' | 'resolved';

	type StackItemByVariantPushConfig<
		Variant extends string,
		UserComponent extends Component<any>,
	> = StackItemCommonConfig<Variant, UserComponent> & {
		props?: Omit<ComponentProps<UserComponent>, 'item'>;
	};

	type StackItemCustomPushConfig<UserComponent extends Component<any>> = StackItemCommonConfig<
		'custom',
		UserComponent
	> & {
		component: UserComponent;
		props?: Omit<ComponentProps<UserComponent>, 'item'>;
	};

	type StackItemPopVerboseInput<UserComponent extends Component<any>> = {
		id?: string;
		resolved?: ComponentResolved<UserComponent>;
	};

	interface StackItemRenderActionAttributes {
		onstackitemmount?: (event: CustomEvent<{ item: StackItem<any> }>) => void;
		onstackitemunmount?: (event: CustomEvent<{ item: StackItem<any> }>) => void;
	}
	type StackItemRenderActionReturn = ActionReturn<
		StackItem<any>,
		StackItemRenderActionAttributes
	>;

	export {};
}

//# sourceMappingURL=index.d.ts.map